﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.IO.Ports;          // added to include serial ports
using System.IO;
using DWORD = System.UInt32;

enum cmds
{
    DONE = 0xB0, GET_VER, RESET_FPGA, ERASE_CHIP, ERASE_64k, PROG_PAGE, READ_PAGE, VERIFY_PAGE, GET_CDONE, RELEASE_FPGA
};

namespace iceFUNprog
{
    public partial class Form1 : Form
    {
        static SerialPort USB_PORT;
        byte moduleFound = 0;
        byte[] SerBuf = new byte[300];
        const Int32 PROGSIZE = 1048576;
        byte[] ProgBuf = new byte[PROGSIZE];
        FileStream fs;

        public Form1()
        {
            InitializeComponent();
            USB_PORT = new SerialPort();

            foreach (string s in SerialPort.GetPortNames())        // Get a list of available serial port names.
            {
                comboBox_comport.Items.Add(s);                      // places each "COMx" name into combobox
            }
            comboBox_comport.Text = "Select COM Port";              // printed only at first execution in the combobox

            try { FileName.Text = Application.UserAppDataRegistry.GetValue("FileName").ToString(); }
            catch { FileName.Text = "Select File"; }

        }


        private void comboBox_comport_SelectedIndexChanged(object sender, EventArgs e)
        {
            USB_PORT.Close();                            // close any existing handle
            USB_PORT.PortName = comboBox_comport.Text;    // retrieves "COMx" from selection in combo box
            USB_PORT.Parity = 0;
            USB_PORT.BaudRate = 19200;
            USB_PORT.StopBits = StopBits.Two;
            USB_PORT.DataBits = 8;
            USB_PORT.ReadTimeout = 5000;
            USB_PORT.WriteTimeout = 5000;
            USB_PORT.Open();

            SerBuf[0] = (byte)cmds.GET_VER;
            transmit(1);
            recieve(2);
            if (SerBuf[0] == 38) textBox1.Text = string.Format("iceFUN Programmer, V{0}", SerBuf[1]);
            else textBox1.Text = string.Format("Not found");
        }

        
        private void transmit(int write_bytes)
        {
            try
            {
                USB_PORT.Write(SerBuf, 0, write_bytes);      // writes specified amount of SerBuf out on COM port
            }
            catch (Exception)
            {
            }
        }


        private void recieve(int read_bytes)
        {
            int x;
            
            for (x = 0; x < read_bytes; x++)       // this will call the read function for the passed number times, 
            {                                      // this way it ensures each byte has been correctly recieved while
                try                                // still using timeouts
                {
                    USB_PORT.Read(SerBuf, x, 1);     // retrieves 1 byte at a time and places in SerBuf at position x
                }
                catch (Exception)                   // timeout or other error occured, set lost comms indicator
                {
                    SerBuf[0] = 255;
                    moduleFound = 0;
                }
            }
        }


        private void button1_Click(object sender, EventArgs e)          // select file
        {
            OpenFileDialog fdlg = new OpenFileDialog();
            fdlg.Title = "Select  File to Program.";
            fdlg.InitialDirectory = @"c:\";
            fdlg.Filter = "Bin files (*.bin)|*.bin|All files (*.*)|*.*";
            fdlg.FilterIndex = 2;
            fdlg.RestoreDirectory = true;
            if (fdlg.ShowDialog() == DialogResult.OK)
            {
                FileName.Text = fdlg.FileName;
                Application.UserAppDataRegistry.SetValue("FileName", fdlg.FileName);
            }
        }

// Verify the file already loaded in ProgBuf
        private void verify(int len)
        {
            int addr = 0;
            textBox2.Text += string.Format("Verifying ");
            textBox2.Refresh();
            int cnt = 0;
            while (addr < len)
            {
                SerBuf[0] = (byte)cmds.VERIFY_PAGE;
                SerBuf[1] = (byte)(addr >> 16);
                SerBuf[2] = (byte)(addr >> 8);
                SerBuf[3] = (byte)addr;
                for (int x = 0; x < 256; x++) SerBuf[x + 4] = ProgBuf[addr++];
                transmit(260);
                recieve(4);
                if (SerBuf[0] > 0)
                {
                    textBox2.Text += string.Format("\r\nVerify failed at {0:X06}, {1:X02} expected, {2:X02} read.\r\n", addr - 256 + SerBuf[1] - 4, SerBuf[2], SerBuf[3]);
                    return;
                }
                if (++cnt == 10)
                {
                    cnt = 0;
                    textBox2.Text += ".";
                    textBox2.Refresh();
                }
            }
            textBox2.Text += "\r\n";
        }


        private void button3_Click(object sender, EventArgs e)          // Program FPGA
        {
            timer1.Enabled = false;
            for (int i = 0; i < PROGSIZE; i++) ProgBuf[i] = 0xff;

            try { fs.Close(); }
            catch { }

            try
            {
                fs = File.OpenRead(FileName.Text);
            }
            catch
            {
                textBox2.Text = "Error - Unable to open File!";
                return;
            }
            SerBuf[0] = (byte)cmds.RESET_FPGA;
            transmit(1);
            recieve(3);
            textBox2.Text = string.Format("FPGA reset. \r\nFlash ID = {0:X02} {1:X02} {2:X02}\r\n", SerBuf[0], SerBuf[1], SerBuf[2]);

            int len = (int)fs.Length;
            textBox2.Text += string.Format("Program length 0x{0:X06}\r\n", len);
            fs.Read(ProgBuf, 0, len);
            int erasePages = (len >> 16) + 1;
            for (int page = 0; page < erasePages; page++)
            {
                SerBuf[0] = (byte)cmds.ERASE_64k;
                SerBuf[1] = (byte)page;
                transmit(2);
                textBox2.Text += string.Format("Erasing sector 0x{0:X02}0000\r\n", page);
                textBox2.Refresh();
                recieve(1);
            }
            int addr = 0;
            textBox2.Text += string.Format("Programming ");
            textBox2.Refresh();
            int cnt = 0;
            while (addr < len)
            {
                SerBuf[0] = (byte)cmds.PROG_PAGE;
                SerBuf[1] = (byte) (addr>>16);
                SerBuf[2] = (byte) (addr>>8);
                SerBuf[3] = (byte) addr;
                for (int x = 0; x < 256; x++) SerBuf[x + 4] = ProgBuf[addr++];
                transmit(260);
                recieve(4);
                if (SerBuf[0] != 0)
                {
                    textBox2.Text += string.Format("\r\nProgram failed at {0:X06}, {1:X02} expected, {2:X02} read.\r\n", addr - 256 + SerBuf[1] - 4, SerBuf[2], SerBuf[3]);
                    fs.Close();
                    return;
                }
                if (++cnt == 10)
                {
                    cnt = 0;
                    textBox2.Text += ".";
                    textBox2.Refresh();
                }
            }
            textBox2.Text += "\r\n";
            if (SerBuf[0] == 0)
            {
                verify(len);
                if (SerBuf[0] == 0)
                {
                    SerBuf[0] = (byte)cmds.RELEASE_FPGA;
                    transmit(1);
                    recieve(1);
                    textBox2.Text += "Done.";
                }
            }
            fs.Close();
        }

        private void button4_Click(object sender, EventArgs e)          // Reset FPGA
        {
            timer1.Enabled = false;
            SerBuf[0] = (byte)cmds.RESET_FPGA;
            transmit(1);
            recieve(3);
            textBox2.Text = string.Format("FPGA reset. \r\nFlash ID = {0:X02} {1:X02} {2:X02}\r\n", SerBuf[0], SerBuf[1], SerBuf[2]);
        }

        private void button6_Click(object sender, EventArgs e)          // Run
        {
            SerBuf[0] = (byte)cmds.RELEASE_FPGA;
            transmit(1);
            recieve(1);
            textBox2.Text = string.Format("Running.. .");
            timer1.Enabled = true;
        }

    }
}
